module	VGA_Ctrl2	(	//	Host Side
                        SW,
						oCurrent_X,
						oCurrent_Y,
						oAddress,
						oRequest,
						//	VGA Side

						oVGA_HS,
						oVGA_VS,
						oVGA_SYNC,
						oVGA_BLANK,
						oVGA_CLOCK,
						//	Control Signal
						iCLK,
						iRST_N	,
			            oVGA_V_BLANK,
						MemON
						);
	input SW;
input       MemON ;						
//	Host Side

output		[21:0]	oAddress;
output		[10:0]	oCurrent_X;
output		[10:0]	oCurrent_Y;
output				oRequest;
//	VGA Side

output	reg			oVGA_HS;
output	reg			oVGA_VS;
output				oVGA_SYNC;
output	reg			oVGA_BLANK;
output				oVGA_CLOCK;
output	reg			oVGA_V_BLANK;
//	Control Signal
input				iCLK;
input				iRST_N;	
//	Internal Registers
reg			[10:0]	H_Cont;
reg			[10:0]	V_Cont;

////////////////////////////////////////////////////////////
//	Horizontal	Parameter
parameter	H_FRONT	=	16;
parameter	H_SYNC	=	96;
parameter	H_BACK	=	48;
parameter	H_ACT	=	640;
parameter	H_BLANK	=	H_FRONT+H_SYNC+H_BACK;
parameter	H_TOTAL	=	H_FRONT+H_SYNC+H_BACK+H_ACT;
////////////////////////////////////////////////////////////
//	Vertical Parameter
parameter	V_FRONT	=	11;
parameter	V_SYNC	=	2;
parameter	V_BACK	=	31;
parameter	V_ACT	=	480;
parameter	V_BLANK	=	V_FRONT+V_SYNC+V_BACK;
parameter	V_TOTAL	=	V_FRONT+V_SYNC+V_BACK+V_ACT;
////////////////////////////////////////////////////////////
assign	oVGA_SYNC	=	1'b1;			//	This pin is unused.
//assign	oVGA_BLANK	=	~((H_Cont<H_BLANK)||(V_Cont<V_BLANK));
always @(posedge iCLK)begin
oVGA_BLANK	=	~( (H_Cont<  H_BLANK ) || ( V_Cont < (V_BLANK + 4 ) )); //4
oVGA_V_BLANK=   ~(  V_Cont<  V_BLANK);
end


assign	oVGA_CLOCK	=	iCLK;

assign	oAddress	=	oCurrent_Y*H_ACT+oCurrent_X;
assign	oRequest	=	((H_Cont>=H_BLANK && H_Cont<H_TOTAL)	&&
						 (V_Cont>=V_BLANK && V_Cont<V_TOTAL));
assign	oCurrent_X	=	(H_Cont>=H_BLANK)	?	H_Cont-H_BLANK	:	11'h0	;
assign	oCurrent_Y	=	(V_Cont>=V_BLANK)	?	V_Cont-V_BLANK	:	11'h0	;

//	Horizontal Generator: Refer to the pixel clock
always@(posedge iCLK or negedge iRST_N)
begin
	if(!iRST_N)
	begin
		H_Cont		<=	0;
		oVGA_HS		<=	1;
	end
	else
	begin
		if ( MemON )begin
			H_Cont		<=	(SW)? H_BLANK : 0;//H_BLANK ok
			oVGA_HS		<=	1;	
	    end
	    else
		if(H_Cont < H_TOTAL)
		H_Cont	<=	H_Cont+1'b1;
		else
		H_Cont	<=	0;
		//	Horizontal Sync
		if(H_Cont==H_FRONT-1)			//	Front porch end
		oVGA_HS	<=	1'b0;
		if(H_Cont==H_FRONT+H_SYNC-1)	//	Sync pulse end
		oVGA_HS	<=	1'b1;
	end
end

//	Vertical Generator: Refer to the horizontal sync
always@(posedge oVGA_HS or posedge MemON)
begin
	if(MemON)
	begin
		V_Cont		<=	(SW)? (V_BLANK-3):0; //V_BLANK-1 ok;// V_FRONT+V_SYNC ;ok //(SW)?V_BLANK:0;
		oVGA_VS		<=	1;
	end
	else
	begin

		if(V_Cont < V_TOTAL)
			V_Cont	<=	V_Cont+1'b1;
		else
			V_Cont	<=	0;
		//	Vertical Sync
		if(V_Cont==V_FRONT-1)			//	Front porch end
			oVGA_VS	<=	1'b0;
			else
			if(V_Cont==(V_FRONT+V_SYNC-1))	//	Sync pulse end
				oVGA_VS	<=	1'b1;
	end
end

endmodule
